/* ______________________________________________________________________
 * Copyright (c) 1998, KL GROUP INC.  All Rights Reserved.
 * http://www.klg.com
 *
 * This software is the confidential and proprietary information of KL
 * Group, Inc. ("Confidential Information").  You shall not disclose such
 * Confidential Information and shall use it only in accordance with the
 * terms of the license agreement you entered into with KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
 * OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. KL GROUP SHALL NOT BE LIABLE FOR ANY
 * DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * ______________________________________________________________________
 */

package locators.klgroup;

import suntest.javastar.lib.*;
import java.awt.*;
import java.awt.event.*;
import jclass.higrid.*;
import jclass.datasource.*;
import jclass.datasource.treemodel.*;
/*
 * Non Component Locator for KL Group JClass HiGrid
 * In Javastar record mode, this locator allows identification
 * of the particular cell, heading, or folder that is being clicked.
 * In playback mode, it will translate the description into
 * the appropriate pixel coordinates.
 *
 */
public class KlgHiGrid implements JSNonComponentLocator {

public JSNCLData findObject(Component c, AWTEvent ev) {
    //get the gridArea (Component that accepts the event)
    Class gridArea = null;
    try {
        gridArea = Class.forName("jclass.higrid.GridArea");
    } catch(ClassNotFoundException excpt){excpt.printStackTrace();}

    //If it is a gridArea that was clicked on, get the grid
    if (gridArea.isInstance(c) && ev instanceof MouseEvent) {

        HiGrid grid = (HiGrid)c.getParent();
        MouseEvent me = (MouseEvent)ev;
        int x = me.getX();
        int y = me.getY();
        Point pointClicked = new Point(x, y);

        //CellPosition object contains the point on the grid, and the
rowNode
        CellPosition cellPosition = null;
        cellPosition = grid.getCellPositionFromPoint(pointClicked);


        String isIndicator = "false";
        // Check that we are in a valid cell
        if (cellPosition == null){
            return null;
        }
        // Check whether this cell is a folder
        else if(cellPosition.isIndicator()) {
            System.out.println("This is an indicator cell.");
            isIndicator = "true";
        }

        try {
            // get RowNode (from the RowNode, we can get anything we need)
            RowNode rowNode = cellPosition.getRowNode();

            // determine RowNode's type
            String rowType = null;
            if (rowNode.getRowFormat() instanceof
jclass.higrid.RecordFormat){
                rowType = "RecordFormat";
            }
            else if (rowNode.getRowFormat()
                     instanceof jclass.higrid.HeaderFormat){
                rowType = "HeaderFormat";
            }
            else { // clicked on a row we are not interested in
                return null;
            }

            // get column index
            int colIndex = cellPosition.getColumnIndex();

            // get row index
            DataTableModel dataTableModel = null;

            long rowBookmark = 0;
            if (rowType.equals("RecordFormat")){
                dataTableModel = rowNode.getDataTableModel();
                rowBookmark = rowNode.getBookmark();
            }
            else { // if HeaderFormat row, use next row's bookmark
                RowNode nextRowNode = (RowNode) rowNode.getNextSibling();
                dataTableModel = nextRowNode.getDataTableModel();
                rowBookmark = nextRowNode.getBookmark();
            }
            int rowIndex = dataTableModel.getRowIndex(rowBookmark);

            // get the table name
            MetaDataModel metaDataModel = dataTableModel.getMetaData();
            String tableName = metaDataModel.getDescription();

            // get the result data
            String colName = metaDataModel.getColumnIdentifier(colIndex);
            Object data = dataTableModel.getResultData(rowBookmark,
colName);

            // get the root row index
            long rootBookmark =
                dataTableModel.getRootAncestorBookmark(rowBookmark);
            DataTableModel rootDataTableModel =
                (DataTableModel)grid.getDataModel().
                getDataTableTree().getRoot();
            int rootRowIndex = rootDataTableModel.getRowIndex(rootBookmark);

            Point point = new Point(x, y);

            return new JSNCLData(isIndicator +
                                 "," +
                                 rowType +
                                 "," +
                                 rootRowIndex +
                                 "," +
                                 tableName +
                                 "," +
                                 rowIndex +
                                 "," +
                                 colIndex,
                                 point,
                                 data);
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }
    // else, we have not clicked on a grid
    System.out.println("Cell not recognized.");
    return null;
}

public JSNCLData getNamedObjectData(Component c, String name) {

    Class gridArea = null;
    try {
        gridArea = Class.forName("jclass.higrid.GridArea");
    }
    catch(ClassNotFoundException excpt){
        excpt.printStackTrace();
    }
    if (gridArea.isInstance(c)) {

        // Parse name :
        String isIndicator;
        String rowType;
        int rootRowIndex;
        String tableName;
        int rowIndex;
        int colIndex;
        int commaIndex = name.indexOf(',');
        if(commaIndex <= 0){
            return null;
        }
        else {
            // parse isIndicator
            isIndicator = name.substring(0, commaIndex);

            // parse rowType
            int nextIndex = name.indexOf(',', commaIndex + 1);
            rowType = name.substring(commaIndex + 1, nextIndex);
            commaIndex = nextIndex;
            nextIndex = name.indexOf(',', commaIndex + 1);

            // parse rootRowIndex
            nextIndex = name.indexOf(',', commaIndex + 1);
            rootRowIndex = Integer.parseInt(name.substring(
                              commaIndex + 1, nextIndex));
            commaIndex = nextIndex;
            nextIndex = name.indexOf(',', commaIndex + 1);

            // parse tableName
            nextIndex = name.indexOf(',', commaIndex + 1);
            tableName = name.substring(commaIndex + 1, nextIndex);
            commaIndex = nextIndex;
            nextIndex = name.indexOf(',', commaIndex + 1);

            // parse rowIndex
            rowIndex = Integer.parseInt(
                         name.substring(commaIndex + 1, nextIndex));
            commaIndex = nextIndex;

            // parse colIndex
            colIndex = Integer.parseInt(
                         name.substring(commaIndex+1));
        }

        try{
            // get HiGrid, its data model, and its data trees
            HiGrid grid = (HiGrid) c.getParent();
            DataModel dataModel = grid.getDataModel();
            TreeModel metaDataTree = dataModel.getMetaDataTree();
            TreeModel dataTableTree = dataModel.getDataTableTree();

            // get MetaData for the target table
            MetaDataModel targetTableMetaData =
                    dataModel.getMetaData(tableName);

            // get the target data table
            DataTableModel targetDataTable;
            if (tableName.equals(
                   // special case if target is the root table
                   ((MetaDataModel)(metaDataTree.getRoot())).
                    getDescription())) {

                targetDataTable = (DataTableModel) dataTableTree.getRoot();
            }
            else {
                // get DataTable for the root table
                DataTableModel rootDataTable =
                    (DataTableModel) dataTableTree.getRoot();

                // get root bookmark
                long rootBookmark = rootDataTable.
                    getRowIdentifier(rootRowIndex);

                // use above data to get the target data table
                targetDataTable = (DataTableModel) rootDataTable.getTable(
                         rootBookmark, (TreeNode) targetTableMetaData);
            }

            // get unique bookmark for row in target table
            long rowBookmark = targetDataTable.getRowIdentifier(rowIndex);

            // get RowNode corresponding to bookmark
            RowNode rowNode = grid.getRowNode(rowBookmark);
            if (rowType.equals("HeaderFormat")){
                rowNode = (RowNode) rowNode.getPreviousSibling();
            }

            // get CellPosition
            CellPosition cellPosition;
            if (isIndicator.equals("true")){
                cellPosition = new CellPosition(
                           rowNode, colIndex, true);
            }
            else {
                cellPosition = new CellPosition(
                           rowNode, colIndex, false);
            }

            // get pixel location from cell position
            Point point = grid.getPointFromCellPosition(cellPosition);

            // compute offset needed to click the middle of cell
            Rectangle cellDrawingArea = cellPosition.getCellFormat().
                getDrawingArea();
            int xOffset = (cellDrawingArea.width / 2);
            int yOffset = (cellDrawingArea.height / 2);
            point.x = point.x + xOffset;
            point.y = point.y + yOffset;

            // get result data
            Object resultData = targetDataTable.getResultData(
                rowBookmark,
targetTableMetaData.getColumnIdentifier(colIndex));

            return new JSNCLData(name, point, resultData);

        }
        catch(Exception e){
            e.printStackTrace();
        }
    }
    //not a grid, or a badly formed locator
    return null;
}
}
