/*
 *  @version 	        @(#)ListComponent.java	1.6 98/05/15
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package locators.klgroup.utils;

import suntest.javastar.lib.*;
import locators.utils.Parser;
import java.awt.*;
import java.awt.event.*;
import jclass.bwt.*;
import jclass.util.JCString;

/*
 * Non Component Locator for JCList of KL Group JClass toolkit
 */
public class ListComponent implements JSNonComponentLocator {

  public  JSNCLData findObject(Component c, AWTEvent ev) {
    if ((c instanceof JCListComponent) && (ev instanceof MouseEvent)) {
      MouseEvent me = (MouseEvent) ev;
      Event oldEvent =
	new Event(ev.getSource(), me.getWhen(), ev.getID(),
		  me.getX(), me.getY(), 0,
		  (me.getModifiers() & ~InputEvent.BUTTON1_MASK));
      return findObject((JCListComponent)c, oldEvent);
    }
    return null;
  }

  public static JSNCLData findObject(JCListComponent jl, Event oldEvent) {
    int which = jl.eventToRow(oldEvent, true);
    if (which == BWTEnum.NOTFOUND)
      return null;

    return getListDat(jl,which);
  }

  public JSNCLData getNamedObjectData(Component c,String name) {
    if ((c instanceof JCListComponent) && name != null) {
      return getNamedObjectData((JCListComponent)c, name);
    }
    return null;
  }

  public static JSNCLData getNamedObjectData(JCListComponent jl,String name) {
    Parser P = new Parser(name);
    String[] items = jl.getItemsStrings();

    if(!P.Valid) return null;

    int idx = P.Index;
    String rname = P.Value;
    if(idx > -1) {
      if (idx < jl.countItems()) {
	if(rname!=null) {
	  if(!rname.equals(Parser.cleanToString(items[idx])))
	    return null;
	}
	return getListDat(jl,idx);
      }
    } else {
      for(int i=0;i<jl.countItems();i++) {
	String tmp = Parser.cleanToString(items[i]);
	if(tmp.equals(rname)){
	  return getListDat(jl,i);
	}
      }
    }
    return null;
  }

  static JSNCLData getListDat(JCListComponent jl, int idx){
    if (idx >= jl.countItems()) return null;

    // compute x and y
    Rectangle rec = jl.getBounds(idx, true);
    if (rec == null) return null;
    Object obj = jl.getItem(idx);
    String name = null;
    if (obj != null && obj instanceof JCString) {
      name = ((JCString)obj).getString();
      if (name.length() == 0)
	name = Parser.cleanToString(obj);
    } else
      name = Parser.cleanToString(obj);

    return new JSNCLData(idx + ":" + name, new Point(rec.x,rec.y), obj);
  }
}
