/*
 *  @version 	        @(#)newOutliner.java	1.11 98/08/07
 *
 * Copyright (c) 1997-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package locators.klgroup.utils;

import suntest.javastar.lib.*;
import java.awt.*;
import java.awt.event.*;
import jclass.bwt.*;
import jclass.util.JCVector;
import locators.klgroup.utils.*;
import locators.utils.Parser;

/*
 * Non Component Locator for JCOutliner of KL Group JClass toolkit
 */
public class newOutliner implements JSNonComponentLocator {

  public JSNCLData findObject(Component c, AWTEvent ev) {
    if ((c instanceof JCOutlinerComponent) && (ev instanceof MouseEvent)) {
      MouseEvent me = (MouseEvent)ev;
      Event nev = new Event(ev.getSource(), me.getWhen(), ev.getID(),
                            me.getX(), me.getY(), 0,
                            (me.getModifiers() & ~InputEvent.BUTTON1_MASK));
      return findObject((JCOutlinerComponent)c, nev);
    }
    return null;
  }

  public static JSNCLData findObject(JCOutlinerComponent jz, Event nev) {
    JCOutlinerNode n = jz.eventToNode(nev, false);
    return nodeData(jz, n);
  }	

  public JSNCLData getNamedObjectData(Component c, String name) {
    if ((c instanceof JCOutlinerComponent) && name != null) {
      return getNamedObjectData((JCOutlinerComponent)c, name);
    }
    return null;
  }

  public static JSNCLData getNamedObjectData(JCOutlinerComponent jz, String name) {
    Parser P = new Parser(name);
    if(!P.Valid) return null;

    int idx = P.Index;
    String rname = P.Value;
    JCVector visibleNodes = jz.getVisibleNodes ();
    if (idx >= 0 ) {
      if (idx >= visibleNodes.size()) return null;

      JCOutlinerNode jcnode = (JCOutlinerNode)visibleNodes.elementAt(idx);
      if (jcnode != null) {
        if (rname == null) return nodeData(jz, jcnode);
        if (rname.equals(nodeName(jcnode)))
          return nodeData(jz, jcnode);
      }
    } else {
      for (int i=0; i< visibleNodes.size(); i++) {
        JCOutlinerNode jcnode = (JCOutlinerNode)visibleNodes.elementAt(i);
        if (jcnode != null && rname.equals(nodeName(jcnode))) {
          return nodeData(jz, jcnode);
        }
      }
    }

    return null;
  }

  private final static String separator = "##";

  private static String getName(JCOutlinerNode n) {
    try {
      return n.getLabel().toString();
    } catch (Throwable e) {
      try {
        return n.getLabelString();
      } catch (Throwable ex) {
          // getUserData is the user object
        Object obj = n.getUserData();
	
          // The best way is to use the method (either Text or Label)
          // of the object to get it's name. But since we don't know
          // the type of the user object, we simplely using it's
          // toString method.
        return obj.toString();
      }
    }
  }

  private static String nodeName(JCOutlinerNode n) {
    String result =  Parser.cleanToString(getName(n));
    JCOutlinerFolderNode parent = n.getParent();
    while (parent != null) {
      result = Parser.cleanToString(getName(parent)) + separator + result;
      parent = parent.getParent();
    }
    return result;
  }

  private static JSNCLData nodeData(JCOutlinerComponent jco, JCOutlinerNode n) {
    if (jco == null || n == null) return null;

    JCVector visibleNodes = jco.getVisibleNodes ();
    for (int i = 0; i< visibleNodes.size(); i++) {
      if (n == (JCOutlinerNode)visibleNodes.elementAt(i)) {
          // get the coordinate of the upper-left corner of the non-component
        Rectangle rec = new Rectangle();
        Rectangle junk = jco.getBounds(n, rec);
        Point p = new Point(rec.x,rec.y);

          // compose the name
        String name = i + ":" + nodeName(n);

          // get the object behind the non-component
        Object o = null;
        try {
          o = n.getUserData();
        } catch (Throwable e) { }

          // decide what to return for the object behind the non-component
        if (n instanceof JCOutlinerFolderNode ||  o == null ||
            o instanceof Integer) {
          return new JSNCLData(name, p, n);
        } else {
          return new JSNCLData(name, p, o);
        }
      }
    }
    return null;
  }
}
