/*
 *  @version 	        @(#)BongoInfo.java	1.2 98/05/15
 *
 * Copyright (c) 1997-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package locators.utils;
import marimba.gui.Widget;

/**
 *  class for processing widget list.  The data structure provides fields
 *  to store all information about objects.
 */

public class BongoInfo  {
  /** the actual bongo widget */
  public Widget widget;

  /** the tag of the parent */
  public String parent;

  /** the local tag name if the name is a caption or an internal name
   * and it is unique throughout the application.
   */
  public String localTagName;

  /** the full tag name with the parent hierarchy */
  public String fullTagName;

  /** the internal name of the object*/
  public String intName;

  /** the caption of the object if exists */
  public String caption;

  /** the actual tag to use */
  public String theTag;

  /** the class name without the package */
  public String className;

  /** the index of the classes relative to the parent */
  public int index;

  public BongoInfo(Widget w,String parent) {
    this(w,parent,"","","","","","",-1);
  }

  public BongoInfo(Widget widget,String parent,String localTagName,String fullTagName,String intName,String caption,String theTag,String className,int index) {
    this.widget=widget;
    this.parent=parent;
    this.localTagName=localTagName;
    this.fullTagName=fullTagName;
    this.intName=intName;
    this.parent=parent;
    this.caption=caption;
    this.theTag=theTag;
    this.className=className;
    this.index=index;
  }

  /** sets the tag based upon precedence.  IF an internal name is
   * available that is used.  If a caption is available that will be
   * used.  If the caption is unique throughout the application it
   * is used, otherwise the full tag is used.
   */
  public void setTheTag() {
    if (intName!=null) {
      theTag=intName;
    } else {
      if (localTagName!=null) {
	theTag=localTagName;
      } else
	theTag=fullTagName;
    }
  }
  public String toString() {
    return "BongoInfo: widget="+widget+" tag="+theTag+" intName="+intName+" caption="+caption+" index="+index;
  }
}
