/*
 *  @version 	        @(#)CafeSpy.java	1.2 98/06/19
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package symantec.itools.awt;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**	
 *
 *
 * @version 	      @(#)CafeSpy.java	1.2 98/06/19
 * @author          Roongko Doong
 *
 */
public class CafeSpy {

  public static TreeNode getTreeNode(TreeView tree, int idx) {
    if (tree.rootNode==null) throw new ArrayIndexOutOfBoundsException();
    if (idx < 0 || idx > tree.getViewCount())
      throw new ArrayIndexOutOfBoundsException();

    Vector nodesVector = new Vector(tree.getCount());
    tree.vectorize(tree.rootNode, false, nodesVector);
    return (TreeNode)nodesVector.elementAt(idx);
  }

  public static int nodeClicked(TreeView tree, int y) {
    int index = (y / tree.cellSize) + tree.sbVPosition;

		//If clicked below the last node
		if (index < 0 || index > tree.getViewCount())
      throw new ArrayIndexOutOfBoundsException();

    return index;
  }

  public static Point nodeCorner(TreeView tree, int idx) {
		if (idx < 0 || idx >= tree.getViewCount())
      throw new ArrayIndexOutOfBoundsException();

    TreeNode newNode = (TreeNode)tree.v.elementAt(idx);
		int newDepth = newNode.getDepth();

    return new Point(tree.cellSize*newDepth + tree.cellSize/4,
                     (idx-tree.sbVPosition)*tree.cellSize + tree.clickSize/2);
  }

  public static int columnClicked(MultiList ml, int x) {
    for (int i = 0; i < ml.headings.length; i++) {
      if ((x > ml.splitters[i] - ml.sbHPosition) && (x < ml.splitters[i+1] - ml.sbHPosition))
        return i;
    }

    throw new ArrayIndexOutOfBoundsException();
  }

  public static int rowClicked(MultiList ml, int y) {
      //Calc row clicked in
    int clickedInRow = ((int) (y - ml.headingHeight) / ml.cellHeight) +
      ml.topRow;

			//Range check the row that was clicked in
			if (clickedInRow < 0 || clickedInRow > ml.cells.rows() - 1)
        throw new ArrayIndexOutOfBoundsException();

      return clickedInRow;
  }

  public static int columnCorner(MultiList ml, int idx) {
    if (idx < ml.headings.length) {
      return ml.splitters[idx] - ml.sbHPosition + 1;
    }
    throw new ArrayIndexOutOfBoundsException();
  }

  public static int rowCorner(MultiList ml, int idx) {
    if (idx < 0 || idx > ml.cells.rows() - 1)
        throw new ArrayIndexOutOfBoundsException();

    return ((idx - ml.topRow) * ml.cellHeight) + ml.headingHeight;
  }

  public static int headingHeight(MultiList ml) {
    return ml.headingHeight;
  }

  public static Point whichTabPoly(TabPanel tab, int idx) {
    int sizeR = tab.vPolys.size();
    if (idx < sizeR) {
      Polygon p;
      p = (Polygon) tab.vPolys.elementAt(idx);
      if (p != tab.nullPoly) {
        Rectangle rec = p.getBounds();
        return new Point(rec.x, rec.y);
      }
    }
    throw new ArrayIndexOutOfBoundsException();
  }

  public static int whichTab(TabPanel tab, int ex, int ey) {
    int sizeR = tab.vPolys.size();
    Polygon p;
    for (int x = 0; x < sizeR; x++) {
      p = (Polygon) tab.vPolys.elementAt(x);
      if ( (p != tab.nullPoly) && p.contains(ex, ey)) {
        return x;
      }
    }
    throw new ArrayIndexOutOfBoundsException();
  }

  public static int mouseCalcIndex(ImageListBox list, int y) {
    return list.mouseCalcIndex(y);
  }

}
